@@ -187,7 +187,24 @@ class Storage(driver.Base):
     @lru.remove
     def remove(self, path):
         mpath = self.create_manta_path(path)
-        self.client.delete_object(mpath)
+        type = self.client.type(mpath)
+        if type != 'directory':
+            return self.client.delete_object(mpath)
+
+        files = self.client.list_directory(mpath)
+        for entry in files:
+            name = mpath + '/' + entry['name']
+            if entry['type'] == 'directory':
+                try:
+                    self.client.delete_directory(name)
+                except Exception:
+                    files += self.client.list_directory(name)
+                    files += [name]
+            else:
+                self.client.delete_object(name)
+        else:
+            return self.client.delete_directory(mpath)
+        self.remove(path)
 
     def content_redirect_url(self, path):
         return self.config.url + path
